@@ -121,7 +121,11 @@ sub process {
   pos($this->{md}) = 0;
 
   # https://spec.commonmark.org/0.30/#characters-and-lines
-  $this->{md} =~ s/\000/\xfffd/g;
+  # TODO: The spec asks for this, however we can’t apply it, because md is a
+  # reference to the value passed by the user and we don’t want to modify it (it
+  # may even be a read-only value). I’m too lazy to find another place to
+  # implement this behavior.
+  # $this->{md} =~ s/\000/\xfffd/g;
 
   # https://spec.commonmark.org/0.30/#tabs
   # TODO: nothing to do at this stage.
@@ -322,7 +326,7 @@ sub _parse_blocks {  ## no critic (RequireArgUnpacking)
   $this->{last_line_is_blank} = 0;
 
   _do_atx_heading($this)
-      || _do_setext_heading($this)
+      || ($this->get_use_setext_headings && _do_setext_heading($this))
       # Thematic breaks are described first in the spec, but the setext headings has
       # precedence in case of conflict, so we test for the break after the heading.
       || _do_thematic_break($this)
